package com.ims.eval.schedule;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.*;
import com.ims.eval.entity.*;
import com.ims.eval.expression.*;
import com.ims.eval.service.IEvaluationScoreCountService;
import com.ims.eval.util.MathCalculatorUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ：xugp
 * @date ：Created in 2023/3/15 9:49
 * @description：得分规则计算
 * @modified By：
 * @version: $
 */

@Slf4j
@Component
@EnableScheduling
public class ScoreCalculationSchedule {

	@Autowired
	private OrganizationEvaluationInfoMapper organizationEvaluationInfoMapper;

	@Autowired
	private EvaluationScoringRuleMapper evaluationScoringRuleMapper;

	@Autowired
	private IntervalScoringTableMapper intervalScoringTableMapper;

	@Autowired
	private IndicatorMapper indicatorMapper;

	@Autowired
	private IEvaluationScoreCountService iEvaluatioinScoreCountService;

	/*@Scheduled(cron = "* 10 11 ? * 3")*/
	public boolean doTask(){

		boolean save = true;
		try {
			//加载考评指标明细数据
			QueryWrapper<OrganizationEvaluationInfo> qw = new QueryWrapper<>();
			List<OrganizationEvaluationInfo> organizationEvaluationInfos= organizationEvaluationInfoMapper.selectList(qw);
			if (organizationEvaluationInfos.size() > 0){
				//对考评记录id进行分组
				Map<String, List<OrganizationEvaluationInfo>> collect = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationId));
				for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()){
					//对指标id进行分组
					if (entry.getValue().size() > 0){
						Map<String, List<OrganizationEvaluationInfo>> collectIn = entry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
						for (Map.Entry<String, List<OrganizationEvaluationInfo>> listEntry : collectIn.entrySet()){
							Indicator indicator = indicatorMapper.selectById(listEntry.getKey());
							QueryWrapper<EvaluationScoringRule> e = new QueryWrapper<>();
							if (StringUtils.isNotEmpty(indicator.getIndicatorCode())) {
								e.lambda().eq(EvaluationScoringRule::getIndicatorId,indicator.getIndicatorCode());
							}
							EvaluationScoringRule evaluationScoringRule = evaluationScoringRuleMapper.selectOne(e);
							if (evaluationScoringRule != null){
								String calculation = replace(listEntry.getValue(), evaluationScoringRule);
								log.info(indicator.getIndicatorCode()+"得分为" + calculation);
								//将得分存入考评得分统计表
								EvaluationScoreCount evaluationScoreCount = EvaluationScoreCount.builder()
									.indicatorId(listEntry.getKey())
									.organizationEvaluationId(entry.getKey())
									.score(calculation)
									.build();
								save = iEvaluatioinScoreCountService.save(evaluationScoreCount);
							}
						}
					}
				}

			}else {
				throw new Exception("考评指标明细数据为空！！");
			}

		}catch (Exception e){
			log.info(e.getMessage());
		}
		return save;
	}

	//中间变量替换为数据
	public String replace(List<OrganizationEvaluationInfo> list,EvaluationScoringRule evaluationScoringRule) throws Exception {

		String score = null;
		//判断可用量化的进行变量替换
		if ("是".equals(list.get(0).getIsQuantified())){
			Map<String, Double> doubleMap = list.stream().collect(Collectors.toMap(OrganizationEvaluationInfo::getOptionCode, OrganizationEvaluationInfo::getQuantifiedValue));
			QueryWrapper<IntervalScoringTable> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(evaluationScoringRule.getId())) {
				qw.lambda().eq(IntervalScoringTable::getRuleId,evaluationScoringRule.getId());
			}
			List<IntervalScoringTable> intervalScoringTables = intervalScoringTableMapper.selectList(qw);
			for (Map.Entry<String, Double> entry : doubleMap.entrySet()){
				intervalScoringTables.stream().forEach(r ->{
					r.setRange(r.getRange().replaceAll(entry.getKey(), entry.getValue() + ""));
					r.setRegularExpression(r.getRegularExpression().replaceAll(entry.getKey(),entry.getValue() + ""));
				});
			}
			//针对只有一种区间评分
			if (intervalScoringTables.size() == 1){
				String regularExpression = intervalScoringTables.get(0).getRegularExpression();
				//公式计算得分
				score = MathCalculatorUtil.handle(regularExpression.trim());
			}
			else {
				//针对多区间评分
				for (IntervalScoringTable intervalScoringTable : intervalScoringTables){
					String range = intervalScoringTable.getRange();
					if (rangeHandle(range)){
						score =  MathCalculatorUtil.handle(intervalScoringTable.getRegularExpression().trim());
						break;
					}
				}
			}
		}else {
			throw new Exception("该指标是非量化指标！");
		}
		return score;
	}


	/*
	 *  判断是否在该区间
	 * */
	public Boolean rangeHandle(String range) {

		Boolean s = true;
		try {
			//将区间表达式拆分
			ArrayList<Token> tokens = Analyzer.getTokens(range);
			//使用不同编码代替函数  支持三级函数嵌套
			AlarmExpression alarmExpression = Analyzer.createAlarmExpression(tokens);
			if (alarmExpression.getVarList() != null && alarmExpression.getVarList().size() > 0) {
				List<AlarmExpression> alarmExpressions = alarmExpression.getChildren();
				for (AlarmExpression alarmExpression1 : alarmExpressions){
					if (alarmExpression1.getChildren().size() > 0 && alarmExpression1.getChildren() != null){
						for (AlarmExpression alarmExpression2 : alarmExpression1.getChildren()){
							if (alarmExpression2.getChildren().size() > 0 && alarmExpression2.getChildren() != null){
								List<AlarmExpression> al = alarmExpression2.getChildren();
								for (AlarmExpression alarmExpression3 : al){
									String calculator = MathCalculatorUtil.calculator(alarmExpression3.getExpression());
									alarmExpression2.setExpression(alarmExpression2.getExpression().replaceAll(alarmExpression3.getFunCode(), calculator));
								}
							}
							String calculator = MathCalculatorUtil.calculator(alarmExpression2.getExpression());
							alarmExpression1.setExpression(alarmExpression1.getExpression().replaceAll(alarmExpression2.getFunCode(), calculator));
						}
						String calculator = MathCalculatorUtil.calculator(alarmExpression1.getExpression());
						alarmExpression.setExpression(alarmExpression.getExpression().replaceAll(alarmExpression1.getFunCode(), calculator));
					}else {
						String calculator = MathCalculatorUtil.calculator(alarmExpression1.getExpression());
						alarmExpression.setExpression(alarmExpression.getExpression().replaceAll(alarmExpression1.getFunCode(), calculator));
					}
				}
				Map<String,Object> map = new HashMap<>();
				s = (Boolean) ScriptShell.parseExpr(alarmExpression.getExpression(), map);
			}
		}catch (Exception e){
			log.info(e.getMessage());
		}
		return s;
	}



	public static void main(String[] args) {


		String s2 = "if(if(2>0,max(1,2)+20,min(1,3)+10)>11,12,if(2<0,min(5,4)+1,max(1,3)+2))";

		String s4 = "if(2-1<0,-(2-1)/min(2-1,4-1)+5,(2-1)/max(2-1,4-1)+5)";
		String s5 = "if(1>0,2,3)==0";
		String s3 = "if(abs(99)<=5%,if(abs(99)>=1%,(5%-abs(99))/1%*0.25+5,6),if(abs(99)<=10%,(5%-abs(99))/1%*0.2+5,4))+if(99==0,5,if(99>0,6,4))";
		/*String sd = "max(1,min(1,2))";
		String sc = MathCalculatorUtil.calculator(sd);*/
		String ss = "if(max(1,3)<1,1,2)";
		ArrayList<Token> tokens = Analyzer.getTokens(s5);

		//使用不同编码代替函数
		AlarmExpression alarmExpression = Analyzer.createAlarmExpression(tokens);
		if (alarmExpression.getVarList() != null && alarmExpression.getVarList().size() > 0) {
			List<AlarmExpression> alarmExpressions = alarmExpression.getChildren();
			for (AlarmExpression alarmExpression1 : alarmExpressions){
				if (alarmExpression1.getChildren().size() > 0 && alarmExpression1.getChildren() != null){
					for (AlarmExpression alarmExpression2 : alarmExpression1.getChildren()){
						if (alarmExpression2.getChildren().size() > 0 && alarmExpression2.getChildren() != null){
							List<AlarmExpression> al = alarmExpression2.getChildren();
							for (AlarmExpression alarmExpression3 : al){
								String calculator = MathCalculatorUtil.calculator(alarmExpression3.getExpression());
								alarmExpression2.setExpression(alarmExpression2.getExpression().replaceAll(alarmExpression3.getFunCode(), calculator));
							}
						}
						String calculator = MathCalculatorUtil.calculator(alarmExpression2.getExpression());
						alarmExpression1.setExpression(alarmExpression1.getExpression().replaceAll(alarmExpression2.getFunCode(), calculator));
					}
					String calculator = MathCalculatorUtil.calculator(alarmExpression1.getExpression());
					alarmExpression.setExpression(alarmExpression.getExpression().replaceAll(alarmExpression1.getFunCode(), calculator));
				}else {
					String calculator = MathCalculatorUtil.calculator(alarmExpression1.getExpression());
					alarmExpression.setExpression(alarmExpression.getExpression().replaceAll(alarmExpression1.getFunCode(), calculator));
				}
			}
		}
		Map<String,Object> map = new HashMap<>();
		Object o = ScriptShell.parseExpr(alarmExpression.getExpression(), map);
		System.out.println(o);
		//判断表达式返回结果
//		String calculator = MathCalculatorUtil.calculator(alarmExpression.getExpression());
//		System.out.println(calculator);

	}

}
